package com.googlecode.webworkwebflow;

import java.util.HashMap;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.webflow.action.AbstractAction;
import org.springframework.webflow.execution.Event;
import org.springframework.webflow.execution.RequestContext;

import com.opensymphony.xwork.ActionContext;
import com.opensymphony.xwork.ActionInvocation;
import com.opensymphony.xwork.ActionProxy;
import com.opensymphony.xwork.ActionProxyFactory;
import com.opensymphony.xwork.util.OgnlValueStack;
import com.opensymphony.xwork.util.TextParseUtil;


/**
 * Allows webwork actions to be executed as Spring Webflow actions.
 */
public class WebworkFlowAdapter extends AbstractAction {
	private static final Log log = LogFactory.getLog(WebworkFlowAdapter.class);

	/**
	 * Constant for the action's parameters.
	 */
	public static final String REQUEST_CONTEXT = "com.opensymphony.xwork.ActionContext.context";

	/**
	 * @see org.springframework.webflow.action.AbstractAction#doExecute(org.springframework.webflow.RequestContext)
	 */
	protected Event doExecute(RequestContext context) throws Exception {
		WebworkExternalContext webworkContext = (WebworkExternalContext) context
				.getExternalContext();
		ActionInvocation invocation = webworkContext.getActionInvocation();
		ActionProxy proxy = invocation.getProxy();

		// Assume the current state id is the name of the webwork action defined
		// in the xwork.xml
		String actionName = context.getCurrentState().getId();

		// Assume the current namespace
		String namespace = invocation.getProxy().getNamespace();

		OgnlValueStack stack = ActionContext.getContext().getValueStack();
		String finalNamespace = TextParseUtil.translateVariables(namespace,
				stack);
		String finalActionName = TextParseUtil.translateVariables(actionName,
				stack);
		String finalMethodName = null;

		HashMap extraContext = new HashMap();
		extraContext.put(ActionContext.VALUE_STACK, ActionContext.getContext()
				.getValueStack());
		extraContext.put(ActionContext.PARAMETERS, ActionContext.getContext()
				.getParameters());
		extraContext.put(REQUEST_CONTEXT, context);

		if (log.isDebugEnabled()) {
			log.debug("Chaining to action " + finalActionName);
		}

		proxy = ActionProxyFactory.getFactory().createActionProxy(namespace, actionName, extraContext);
		proxy.setExecuteResult(false);
		if (null != finalMethodName) {
			proxy.setMethod(finalMethodName);
		}
		proxy.execute();
		String resultCode = proxy.getInvocation().getResultCode();
		return result(resultCode);
	}
}
